//
//  XETableViewController.m
//  XingSDK-Example
//
//  Created by Johannes Plunien on 12/18/12.
//  Copyright (c) 2012 XING AG. All rights reserved.
//

#import "UIScrollView+SVInfiniteScrolling.h"
#import "UIScrollView+SVPullToRefresh.h"
#import "XETableViewController.h"
#import "XEUtil.h"

@interface XETableViewController ()

@property (nonatomic, strong) NSFetchedResultsController *fetchedResultsController;

@end

@implementation XETableViewController

- (id)init {
    return [self initWithUserID:XingSDK.session.xingUserID];
}

- (id)initWithUserID:(NSString *)userID {
    return [self initWithUser:[XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID]];
}

- (id)initWithUser:(XingEntityUser *)inUser {
    if (inUser == nil) {
        [XEUtil logout];
        return nil;
    }
    if (self = [super init]) {
        _user = inUser;
    }
    return self;
}

#pragma mark - View life cycle

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupInfiniteScrolling];
    [self setupPullToRefresh];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    if (self.fetchedResultsController == nil) return;
    int entityCount = [[self entityClass] MR_countOfEntitiesWithPredicate:self.fetchedResultsController.fetchRequest.predicate];
    if (entityCount == 0) {
        [self.tableView triggerInfiniteScrolling];
    }
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    self.fetchedResultsController = nil;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.fetchedResultsController.sections.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    id<NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsController.sections objectAtIndex:section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    }
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}

#pragma mark NSFetchedResultsControllerDelegate methods

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type {
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;

        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath {
    UITableView *tableView = self.tableView;
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;

        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;

        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;

        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView endUpdates];
}

#pragma mark - Abstract methods

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    [NSException raise:NSGenericException format:@"The configureCell:atIndexPath method must be overridden"];
}

- (void)deleteAllInContext:(NSManagedObjectContext *)context {
    [NSException raise:NSGenericException format:@"The deleteAllInContext: method must be overridden"];
}

- (Class)entityClass {
    [NSException raise:NSGenericException format:@"The entityClass method must be overridden"];
    return [self class];
}

- (NSSet *)items {
    [NSException raise:NSGenericException format:@"The items method must be overridden"];
    return [NSSet set];
}

- (RACSignal *)loadItems {
    [NSException raise:NSGenericException format:@"The loadItems method must be overridden"];
    return [RACSubject subject];
}

#pragma mark - Setup methods

- (void)setupInfiniteScrolling {
    if (self.fetchedResultsController == nil) return;

    __weak __typeof(&*self)weakSelf = self;

    [self.tableView addInfiniteScrollingWithActionHandler:^{
        if (weakSelf.total > 0 && weakSelf.items.count >= weakSelf.total) {
            [weakSelf.tableView.infiniteScrollingView stopAnimating];
            return;
        }
        [[[weakSelf loadItems] deliverOn:RACScheduler.mainThreadScheduler] subscribeCompleted:^{
            [weakSelf.tableView.infiniteScrollingView stopAnimating];
        }];
    }];
}

- (void)setupPullToRefresh {
    if (self.fetchedResultsController == nil) return;

    __weak __typeof(&*self)weakSelf = self;

    [self.tableView addPullToRefreshWithActionHandler:^{
        SEL pullToRefreshStarted = @selector(pullToRefreshStarted);
        SEL pullToRefreshFinished = @selector(pullToRefreshFinished);
        if ([weakSelf respondsToSelector:pullToRefreshStarted]) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
            [weakSelf performSelector:pullToRefreshStarted];
#pragma clang diagnostic pop
        }
        BOOL showsInfiniteScrolling = weakSelf.tableView.showsInfiniteScrolling;
        if (showsInfiniteScrolling) weakSelf.tableView.showsInfiniteScrolling = NO;
        [MagicalRecord saveWithBlock:^(NSManagedObjectContext *localContext) {
            [weakSelf deleteAllInContext:localContext];
        } completion:^(BOOL success, NSError *error) {
            [[[weakSelf loadItems] deliverOn:RACScheduler.mainThreadScheduler] subscribeCompleted:^{
                [weakSelf.tableView.pullToRefreshView stopAnimating];
                if (showsInfiniteScrolling) weakSelf.tableView.showsInfiniteScrolling = YES;;
                if ([weakSelf respondsToSelector:pullToRefreshFinished]) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
                    [weakSelf performSelector:pullToRefreshFinished];
#pragma clang diagnostic pop
                }
            }];
        }];
    }];
}

@end
